/***********************************************************************/
/*                                                                     */
/*  FILE        :Main.c or Main.cpp                                    */
/*  DATE        :Tue, Oct 31, 2006                                     */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :                                                      */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/
/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
//#include "typedefine.h"
#include "iodefine.h"
#include "stdbool.h"
#include "R_PG_RSPI_RX210.h"
#include <string.h>

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define BUFF_SIZE       (4)                         /* setting buffer size */

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/
void main(void);

/*******************************************************************************
* Constant Definition
*******************************************************************************/
const uint32_t send_buf[BUFF_SIZE] =
{
    0x11, 0x22, 0x33, 0x44
};  /* send data from RSPI0 to SCI5 */

/*******************************************************************************
* Private global variables and functions
*******************************************************************************/
uint32_t CH0_read_buf[BUFF_SIZE];                   /* read buffer for RSPI0 */
uint32_t dummy_rcv[BUFF_SIZE];
uint32_t dummy_trs[BUFF_SIZE] = { 0x00, 0x00, 0x00, 0x00 };
uint8_t SCI5_read_buf[BUFF_SIZE];                   /* read buffer for SCI5 */
uint8_t SCI5_dummy_rcv[BUFF_SIZE];
uint8_t SCI5_dummy_trs[BUFF_SIZE] = { 0x00, 0x00, 0x00, 0x00 };

bool rspi0_receive_flag;
bool sci5_receive_flag;
bool over_run0, mode_fault0, parity_error0;
bool over_run_sci5;

/*******************************************************************************
* Function Name: main
* Description  : Main program
* Arguments    : none
* Return Value : none
*******************************************************************************/
void main(void)
{
    /* Receive data from RSPI0 */
    sci5_receive_flag = true;
    R_PG_SCI_SPIMode_Transfer_C5(SCI5_dummy_trs, SCI5_read_buf, BUFF_SIZE);

    /* Send data to SCI5 */
    rspi0_receive_flag = false;
    R_PG_RSPI_StartTransfer_C0((uint32_t *)send_buf, dummy_rcv, 1);

    while(1);
}

/*******************************************************************************
* Function Name: Spi0Intfunc
* Description  : Finish transfer(RSPI0)
* Arguments    : none
* Return Value : none
*******************************************************************************/
void Spi0IntFunc(void)
{
    /* Get error flag and clear */
    R_PG_RSPI_GetError_C0(&over_run0, &mode_fault0, &parity_error0);

    /* error occurred ? */
    if ( over_run0 || mode_fault0 || parity_error0 )
    {
        /* added error function */
    }
    else
    {
        /* RSPI0 is received ? */
        if ( true == rspi0_receive_flag )
        {
            rspi0_receive_flag = false;

            /* send/receive OK ? */
            if ( 0 == strcmp((const char *)CH0_read_buf, (const char *)send_buf) )
            {
                R_PG_IO_PORT_Write_PH2(1);    /* Turn on D3 */
            }
            else
            {
                /* Do nothing */
            }
        }
        else
        {
            /* Do nothing */
        }
    }
}

/*******************************************************************************
* Function Name: Sci5TrFunc
* Description  : Finish transmitting(SCI5)
* Arguments    : none
* Return Value : none
*******************************************************************************/
void Sci5TrFunc(void)
{
    /* SCI5 is transmitted ? */
    if ( false == sci5_receive_flag )
    {
        /* Send data to RSPI0 */
        R_PG_SCI_SPIMode_Transfer_C5(SCI5_read_buf, SCI5_dummy_rcv, BUFF_SIZE);

        /* Receive data from SCI5 */
        rspi0_receive_flag = true;
        R_PG_RSPI_StartTransfer_C0(dummy_trs, CH0_read_buf, 1);
    }
    else
    {
        /* Do nothing */
    }
}

/*******************************************************************************
* Function Name: Sci5ReFunc
* Description  : Finish receiving(SCI5)
* Arguments    : none
* Return Value : none
*******************************************************************************/
void Sci5ReFunc(void)
{
    /* SCI5 is received ? */
    if ( true == sci5_receive_flag )
    {
        sci5_receive_flag = false;
    }
    else
    {
        R_PG_SCI_StopModule_C5();
    }
}

/*******************************************************************************
* Function Name: Sci5ErFunc
* Description  : Detect error(SCI5)
* Arguments    : none
* Return Value : none
*******************************************************************************/
void Sci5ErFunc(void)
{
    /* Get error flag */
    R_PG_SCI_SPIMode_GetErrorFlag_C5(&over_run_sci5);

    /* added error function */
}

/******************************************************************************
Option-Setting Memory
******************************************************************************/
#pragma address OFS1_REG = 0xFFFFFF88   /* OFS1 register */
const unsigned long OFS1_REG = 0xFFFFFFFF;

#pragma address OFS0_REG = 0xFFFFFF8C   /* OFS0 register */
const unsigned long OFS0_REG = 0xFFFFFFFF;
